@@ -1,5 +1,5 @@
 #
-# $Id: LWP.pm,v 1.46 1997/01/29 13:04:41 aas Exp $
+# $Id: LWP.pm,v 1.49 1997/04/05 13:49:42 aas Exp $
 
 package LWP;
 
@@ -8,7 +8,7 @@ require Exporter;
                     # don't export anything.  This will noe be neccessary
                     # when perl5.004 is out.
 
-$VERSION = "5.07";
+$VERSION = "5.09";
 
 sub Version { $VERSION; }
 
@@ -23,6 +23,11 @@ __END__
 
 LWP - Library for WWW access in Perl
 
+=head1 SYNOPSIS
+
+  use LWP;
+  print "This is libwww-perl-$LWP::VERSION\n";
+
 
 =head1 DESCRIPTION
 
